@@ -21,6 +21,7 @@
 
 #include "config.h"
 #include <glib/gi18n-lib.h>
+#include <uuid.h>
 #include <blockdev/blockdev.h>
 
 #include "udiskslogging.h"
@@ -35,18 +36,15 @@
 #include "udisksthreadedjob.h"
 #include "udiskssimplejob.h"
 #include "udisksstate.h"
-#include "udisksfstabmonitor.h"
-#include "udisksfstabentry.h"
 #include "udiskscrypttabmonitor.h"
 #include "udiskscrypttabentry.h"
 #include "udiskslinuxblockobject.h"
 #include "udiskslinuxdevice.h"
 #include "udisksmodulemanager.h"
+#include "udisksmodule.h"
 #include "udisksconfigmanager.h"
-
-#ifdef HAVE_LIBMOUNT
+#include "udiskslinuxmountoptions.h"
 #include "udisksutabmonitor.h"
-#endif
 
 /**
  * SECTION:udisksdaemon
@@ -79,13 +77,8 @@ struct _UDisksDaemon
 
   UDisksState *state;
 
-  UDisksFstabMonitor *fstab_monitor;
-
   UDisksCrypttabMonitor *crypttab_monitor;
-
-#ifdef HAVE_LIBMOUNT
   UDisksUtabMonitor *utab_monitor;
-#endif
 
   UDisksModuleManager *module_manager;
 
@@ -95,6 +88,7 @@ struct _UDisksDaemon
   gboolean force_load_modules;
   gboolean uninstalled;
   gboolean enable_tcrypt;
+  gchar *uuid;
 };
 
 struct _UDisksDaemonClass
@@ -108,7 +102,6 @@ enum
   PROP_CONNECTION,
   PROP_OBJECT_MANAGER,
   PROP_MOUNT_MONITOR,
-  PROP_FSTAB_MONITOR,
   PROP_CRYPTTAB_MONITOR,
   PROP_MODULE_MANAGER,
   PROP_CONFIG_MANAGER,
@@ -116,6 +109,7 @@ enum
   PROP_FORCE_LOAD_MODULES,
   PROP_UNINSTALLED,
   PROP_ENABLE_TCRYPT,
+  PROP_UUID,
 };
 
 G_DEFINE_TYPE (UDisksDaemon, udisks_daemon, G_TYPE_OBJECT);
@@ -126,24 +120,22 @@ udisks_daemon_finalize (GObject *object)
   UDisksDaemon *daemon = UDISKS_DAEMON (object);
 
   udisks_state_stop_cleanup (daemon->state);
-  g_object_unref (daemon->state);
+
+  /* Modules use the monitors and try to reference them when cleaning up */
+  udisks_module_manager_unload_modules (daemon->module_manager);
 
   g_clear_object (&daemon->authority);
   g_object_unref (daemon->object_manager);
   g_object_unref (daemon->linux_provider);
   g_object_unref (daemon->connection);
-
-  /* Modules use the monitors and try to reference them when cleaning up */
-  udisks_module_manager_unload_modules (daemon->module_manager);
   g_object_unref (daemon->mount_monitor);
-  g_object_unref (daemon->fstab_monitor);
   g_object_unref (daemon->crypttab_monitor);
-#ifdef HAVE_LIBMOUNT
   g_object_unref (daemon->utab_monitor);
-#endif
-
   g_clear_object (&daemon->module_manager);
 
+  g_object_unref (daemon->state);
+  g_free (daemon->uuid);
+
   g_clear_object (&daemon->config_manager);
 
   if (G_OBJECT_CLASS (udisks_daemon_parent_class)->finalize != NULL)
@@ -172,10 +164,6 @@ udisks_daemon_get_property (GObject    *object,
       g_value_set_object (value, udisks_daemon_get_mount_monitor (daemon));
       break;
 
-    case PROP_FSTAB_MONITOR:
-      g_value_set_object (value, udisks_daemon_get_fstab_monitor (daemon));
-      break;
-
     case PROP_CRYPTTAB_MONITOR:
       g_value_set_object (value, udisks_daemon_get_crypttab_monitor (daemon));
       break;
@@ -204,6 +192,10 @@ udisks_daemon_get_property (GObject    *object,
       g_value_set_boolean (value, udisks_daemon_get_enable_tcrypt (daemon));
       break;
 
+    case PROP_UUID:
+      g_value_set_string (value, udisks_daemon_get_uuid (daemon));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -217,6 +209,8 @@ udisks_daemon_set_property (GObject      *object,
                             GParamSpec   *pspec)
 {
   UDisksDaemon *daemon = UDISKS_DAEMON (object);
+  const gchar *s;
+  uuid_t uu;
 
   switch (prop_id)
     {
@@ -241,6 +235,17 @@ udisks_daemon_set_property (GObject      *object,
       daemon->enable_tcrypt = g_value_get_boolean (value);
       break;
 
+    case PROP_UUID:
+      s = g_value_get_string (value);
+      if (s != NULL && uuid_parse (s, uu) == 0)
+        {
+          g_free (daemon->uuid);
+          daemon->uuid = g_strdup (s);
+        }
+      else
+        g_warning ("Invalid UUID string '%s'", s);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -261,12 +266,86 @@ mount_monitor_on_mount_removed (UDisksMountMonitor *monitor,
   udisks_state_check (daemon->state);
 }
 
+static gboolean
+load_modules_in_idle_cb (gpointer user_data)
+{
+  UDisksDaemon *daemon = UDISKS_DAEMON (user_data);
+
+  /* clear the state file, loading all modules anyway */
+  udisks_state_clear_modules (daemon->state);
+
+  udisks_module_manager_load_modules (daemon->module_manager);
+
+  return G_SOURCE_REMOVE;
+}
+
+static gboolean
+check_modules_state_in_idle_cb (gpointer user_data)
+{
+  UDisksDaemon *daemon = UDISKS_DAEMON (user_data);
+  gchar **modules;
+  gchar **l;
+  GError *error = NULL;
+
+  modules = udisks_state_get_modules (daemon->state);
+
+  /* first clear the state file */
+  udisks_state_clear_modules (daemon->state);
+
+  if (modules)
+    {
+      if (*modules)
+        g_warning ("Unclean shutdown detected, reloading modules from previous session.");
+
+      for (l = modules; l && *l; l++)
+        if (! udisks_module_manager_load_single_module (daemon->module_manager, *l, &error))
+          {
+            g_warning ("Error re-initializing module %s: %s", *l, error->message);
+            g_clear_error (&error);
+          }
+
+      g_strfreev (modules);
+    }
+
+  return G_SOURCE_REMOVE;
+}
+
+static void
+bd_log_redirect (gint level, const gchar *msg)
+{
+  switch (level)
+    {
+      case BD_UTILS_LOG_EMERG:
+      case BD_UTILS_LOG_ALERT:
+      case BD_UTILS_LOG_CRIT:
+      case BD_UTILS_LOG_ERR:
+        udisks_critical ("[blockdev] %s", msg);
+        break;
+      case BD_UTILS_LOG_WARNING:
+        udisks_warning ("[blockdev] %s", msg);
+        break;
+      case BD_UTILS_LOG_NOTICE:
+        udisks_notice ("[blockdev] %s", msg);
+        break;
+      case BD_UTILS_LOG_INFO:
+        udisks_info ("[blockdev] %s", msg);
+        break;
+      case BD_UTILS_LOG_DEBUG:
+        udisks_debug ("[blockdev] %s", msg);
+        break;
+      default:
+        break;
+    }
+}
+
 static void
 udisks_daemon_constructed (GObject *object)
 {
   UDisksDaemon *daemon = UDISKS_DAEMON (object);
   GError *error;
   gboolean ret = FALSE;
+  gchar uuid_buf[UUID_STR_LEN] = {0};
+  uuid_t uuid;
 
   /* NULL means no specific so_name (implementation) */
   BDPluginSpec part_plugin = {BD_PLUGIN_PART, NULL};
@@ -275,27 +354,23 @@ udisks_daemon_constructed (GObject *object)
   BDPluginSpec mdraid_plugin = {BD_PLUGIN_MDRAID, NULL};
   BDPluginSpec fs_plugin = {BD_PLUGIN_FS, NULL};
   BDPluginSpec crypto_plugin = {BD_PLUGIN_CRYPTO, NULL};
-
+  BDPluginSpec nvme_plugin = {BD_PLUGIN_NVME, NULL};
+#ifdef HAVE_SMART
+  BDPluginSpec smart_plugin = {BD_PLUGIN_SMART, NULL};
+#endif
   /* The core daemon needs the part, swap, loop, mdraid, fs and crypto plugins.
      Additional plugins are required by various modules, but they make sure
      plugins are loaded themselves. */
   BDPluginSpec *plugins[] = {&part_plugin, &swap_plugin, &loop_plugin, &mdraid_plugin,
-                             &fs_plugin, &crypto_plugin, NULL};
+                             &fs_plugin, &crypto_plugin, &nvme_plugin,
+#ifdef HAVE_SMART
+                             &smart_plugin,
+#endif
+                             NULL};
   BDPluginSpec **plugin_p = NULL;
   error = NULL;
 
-  /* Skip runtime dependency checks when initializing libblockdev. Plugin
-     shouldn't fail to load just because some if its dependencies is missing.
-   */
-  ret = bd_switch_init_checks (FALSE, &error);
-  if (!ret)
-    {
-      udisks_error ("Error initializing libblockdev library: %s (%s, %d)",
-                    error->message, g_quark_to_string (error->domain), error->code);
-      g_clear_error (&error);
-    }
-
-  ret = bd_try_init (plugins, NULL, NULL, &error);
+  ret = bd_try_init (plugins, bd_log_redirect, NULL, &error);
   if (!ret)
     {
       if (error)
@@ -306,15 +381,24 @@ udisks_daemon_constructed (GObject *object)
         }
     else
       {
-        /* a missing plugin is okay, calling functions from it will fail, but
-           until that happens, life will just be great */
-        for (plugin_p=plugins; *plugin_p; plugin_p++)
+        for (plugin_p = plugins; *plugin_p; plugin_p++)
           if (!bd_is_plugin_available ((*plugin_p)->name))
-            udisks_warning ("Failed to load the '%s' libblockdev plugin",
-                            bd_get_plugin_name ((*plugin_p)->name));
+            udisks_error ("Failed to load the '%s' libblockdev plugin",
+                          bd_get_plugin_name ((*plugin_p)->name));
       }
     }
 
+#ifdef DEBUG
+  bd_utils_set_log_level(BD_UTILS_LOG_DEBUG);
+#else
+  bd_utils_set_log_level(BD_UTILS_LOG_INFO);
+#endif
+
+  /* Generate global UUID */
+  uuid_generate (uuid);
+  uuid_unparse (uuid, &uuid_buf[0]);
+  daemon->uuid = g_strdup (uuid_buf);
+
   daemon->authority = polkit_authority_get_sync (NULL, &error);
   if (daemon->authority == NULL)
     {
@@ -362,24 +446,32 @@ udisks_daemon_constructed (GObject *object)
                     G_CALLBACK (mount_monitor_on_mount_removed),
                     daemon);
 
-  daemon->fstab_monitor = udisks_fstab_monitor_new ();
   daemon->crypttab_monitor = udisks_crypttab_monitor_new ();
-#ifdef HAVE_LIBMOUNT
   daemon->utab_monitor = udisks_utab_monitor_new ();
-#endif
 
   /* now add providers */
   daemon->linux_provider = udisks_linux_provider_new (daemon);
-
-  if (daemon->force_load_modules
-      || (udisks_config_manager_get_load_preference (daemon->config_manager)
-          == UDISKS_MODULE_LOAD_ONSTARTUP))
-    {
-      udisks_module_manager_load_modules (daemon->module_manager);
-    }
-
   udisks_provider_start (UDISKS_PROVIDER (daemon->linux_provider));
 
+  /* fill in default mount options */
+  g_object_set_data_full (object,
+                          "mount-options",
+                          udisks_linux_mount_options_get_builtin (),
+                          (GDestroyNotify) g_hash_table_destroy);
+
+  /* Load modules if requested but only once providers have started and
+   * have connected on the UDisksModuleManager::modules-activated signal.
+   */
+  if (daemon->force_load_modules ||
+      udisks_config_manager_get_load_preference (daemon->config_manager) == UDISKS_MODULE_LOAD_ONSTARTUP)
+    {
+      /* load all modules */
+      g_idle_add (load_modules_in_idle_cb, daemon);
+    } else {
+      /* crash recovery - load active modules from previous session */
+      g_idle_add (check_modules_state_in_idle_cb, daemon);
+    }
+
   /* Export the ObjectManager */
   g_dbus_object_manager_server_set_connection (daemon->object_manager, daemon->connection);
 
@@ -506,6 +598,20 @@ udisks_daemon_class_init (UDisksDaemonClass *klass)
                                                          G_PARAM_READABLE |
                                                          G_PARAM_WRITABLE |
                                                          G_PARAM_CONSTRUCT_ONLY));
+
+  /**
+   * UDisksDaemon:uuid:
+   *
+   * The UUID specific for this daemon instance.
+   */
+  g_object_class_install_property (gobject_class,
+                                   PROP_UUID,
+                                   g_param_spec_string ("uuid",
+                                                        "Daemon UUID",
+                                                        "The UUID specific for this daemon instance",
+                                                         NULL,
+                                                         G_PARAM_READABLE |
+                                                         G_PARAM_WRITABLE));
 }
 
 /**
@@ -582,21 +688,6 @@ udisks_daemon_get_mount_monitor (UDisksDaemon *daemon)
   return daemon->mount_monitor;
 }
 
-/**
- * udisks_daemon_get_fstab_monitor:
- * @daemon: A #UDisksDaemon
- *
- * Gets the fstab monitor used by @daemon.
- *
- * Returns: A #UDisksFstabMonitor. Do not free, the object is owned by @daemon.
- */
-UDisksFstabMonitor *
-udisks_daemon_get_fstab_monitor (UDisksDaemon *daemon)
-{
-  g_return_val_if_fail (UDISKS_IS_DAEMON (daemon), NULL);
-  return daemon->fstab_monitor;
-}
-
 /**
  * udisks_daemon_get_crypttab_monitor:
  * @daemon: A #UDisksDaemon
@@ -612,7 +703,6 @@ udisks_daemon_get_crypttab_monitor (UDisksDaemon *daemon)
   return daemon->crypttab_monitor;
 }
 
-#ifdef HAVE_LIBMOUNT
 /**
  * udisks_daemon_get_utab_monitor:
  * @daemon: A #UDisksDaemon
@@ -627,7 +717,6 @@ udisks_daemon_get_utab_monitor (UDisksDaemon *daemon)
   g_return_val_if_fail (UDISKS_IS_DAEMON (daemon), NULL);
   return daemon->utab_monitor;
 }
-#endif
 
 /**
  * udisks_daemon_get_linux_provider:
@@ -733,6 +822,7 @@ common_job (UDisksDaemon    *daemon,
             UDisksObject    *object,
             const gchar     *job_operation,
             uid_t            job_started_by_uid,
+            gboolean         no_inhibit,
             gpointer         job)
 {
   gchar *job_object_path;
@@ -743,9 +833,12 @@ common_job (UDisksDaemon    *daemon,
   job_data = g_new0 (JobData, 1);
   job_data->daemon = g_object_ref (daemon);
   /* register inhibitor to systemd logind while job is running */
-  operation_description = udisks_client_get_job_description_from_operation (job_operation);
-  job_data->inhibit_cookie = udisks_daemon_util_inhibit_system_sync (operation_description);
-  g_free (operation_description);
+  if (!no_inhibit)
+    {
+      operation_description = udisks_client_get_job_description_from_operation (job_operation);
+      job_data->inhibit_cookie = udisks_daemon_util_inhibit_system_sync (operation_description);
+      g_free (operation_description);
+    }
 
   if (object != NULL)
     udisks_base_job_add_object (UDISKS_BASE_JOB (job), object);
@@ -774,6 +867,7 @@ common_job (UDisksDaemon    *daemon,
  * @object: (allow-none): A #UDisksObject to add to the job or %NULL.
  * @job_operation: The operation for the job.
  * @job_started_by_uid: The user who started the job.
+ * @no_inhibit: Avoid placing system inhibitor lock.
  * @cancellable: A #GCancellable or %NULL.
  *
  * Launches a new simple job.
@@ -790,6 +884,7 @@ udisks_daemon_launch_simple_job (UDisksDaemon    *daemon,
                                  UDisksObject    *object,
                                  const gchar     *job_operation,
                                  uid_t            job_started_by_uid,
+                                 gboolean         no_inhibit,
                                  GCancellable    *cancellable)
 {
   UDisksSimpleJob *job;
@@ -797,7 +892,7 @@ udisks_daemon_launch_simple_job (UDisksDaemon    *daemon,
   g_return_val_if_fail (UDISKS_IS_DAEMON (daemon), NULL);
 
   job = udisks_simple_job_new (daemon, cancellable);
-  return common_job (daemon, object, job_operation, job_started_by_uid, job);
+  return common_job (daemon, object, job_operation, job_started_by_uid, no_inhibit, job);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -808,6 +903,7 @@ udisks_daemon_launch_simple_job (UDisksDaemon    *daemon,
  * @object: (allow-none): A #UDisksObject to add to the job or %NULL.
  * @job_operation: The operation for the job.
  * @job_started_by_uid: The user who started the job.
+ * @no_inhibit: Avoid placing system inhibitor lock.
  * @job_func: The function to run in another thread.
  * @user_data: User data to pass to @job_func.
  * @user_data_free_func: Function to free @user_data with or %NULL.
@@ -837,6 +933,7 @@ udisks_daemon_launch_threaded_job  (UDisksDaemon          *daemon,
                                     UDisksObject          *object,
                                     const gchar           *job_operation,
                                     uid_t                  job_started_by_uid,
+                                    gboolean               no_inhibit,
                                     UDisksThreadedJobFunc  job_func,
                                     gpointer               user_data,
                                     GDestroyNotify         user_data_free_func,
@@ -852,7 +949,7 @@ udisks_daemon_launch_threaded_job  (UDisksDaemon          *daemon,
                                  user_data_free_func,
                                  daemon,
                                  cancellable);
-  return common_job (daemon, object, job_operation, job_started_by_uid, job);
+  return common_job (daemon, object, job_operation, job_started_by_uid, no_inhibit, job);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -863,6 +960,7 @@ udisks_daemon_launch_threaded_job  (UDisksDaemon          *daemon,
  * @object: (allow-none): A #UDisksObject to add to the job or %NULL.
  * @job_operation: The operation for the job.
  * @job_started_by_uid: The user who started the job.
+ * @no_inhibit: Avoid placing system inhibitor lock.
  * @cancellable: A #GCancellable or %NULL.
  * @run_as_uid: The #uid_t to run the command as.
  * @run_as_euid: The effective #uid_t to run the command as.
@@ -891,6 +989,7 @@ udisks_daemon_launch_spawned_job (UDisksDaemon    *daemon,
                                   UDisksObject    *object,
                                   const gchar     *job_operation,
                                   uid_t            job_started_by_uid,
+                                  gboolean         no_inhibit,
                                   GCancellable    *cancellable,
                                   uid_t            run_as_uid,
                                   uid_t            run_as_euid,
@@ -914,6 +1013,7 @@ udisks_daemon_launch_spawned_job (UDisksDaemon    *daemon,
                                           object,
                                           job_operation,
                                           job_started_by_uid,
+                                          no_inhibit,
                                           cancellable,
                                           run_as_uid,
                                           run_as_euid,
@@ -932,6 +1032,7 @@ udisks_daemon_launch_spawned_job (UDisksDaemon    *daemon,
  * @object: (allow-none): A #UDisksObject to add to the job or %NULL.
  * @job_operation: The operation for the job.
  * @job_started_by_uid: The user who started the job.
+ * @no_inhibit: Avoid placing system inhibitor lock.
  * @cancellable: A #GCancellable or %NULL.
  * @run_as_uid: The #uid_t to run the command as.
  * @run_as_euid: The effective #uid_t to run the command as.
@@ -965,6 +1066,7 @@ udisks_daemon_launch_spawned_job_gstring (
                                   UDisksObject    *object,
                                   const gchar     *job_operation,
                                   uid_t            job_started_by_uid,
+                                  gboolean         no_inhibit,
                                   GCancellable    *cancellable,
                                   uid_t            run_as_uid,
                                   uid_t            run_as_euid,
@@ -986,7 +1088,7 @@ udisks_daemon_launch_spawned_job_gstring (
   job = udisks_spawned_job_new (command_line, input_string, run_as_uid, run_as_euid, daemon, cancellable);
   g_free (command_line);
 
-  return common_job (daemon, object, job_operation, job_started_by_uid, job);
+  return common_job (daemon, object, job_operation, job_started_by_uid, no_inhibit, job);
 }
 
 /* ---------------------------------------------------------------------------------------------------- */
@@ -1031,6 +1133,7 @@ spawned_job_sync_on_completed (UDisksJob    *job,
  * @object: (allow-none): A #UDisksObject to add to the job or %NULL.
  * @job_operation: The operation for the job.
  * @job_started_by_uid: The user who started the job.
+ * @no_inhibit: Avoid placing system inhibitor lock.
  * @cancellable: A #GCancellable or %NULL.
  * @run_as_uid: The #uid_t to run the command as.
  * @run_as_euid: The effective #uid_t to run the command as.
@@ -1050,6 +1153,7 @@ udisks_daemon_launch_spawned_job_sync (UDisksDaemon    *daemon,
                                        UDisksObject    *object,
                                        const gchar     *job_operation,
                                        uid_t            job_started_by_uid,
+                                       gboolean         no_inhibit,
                                        GCancellable    *cancellable,
                                        uid_t            run_as_uid,
                                        uid_t            run_as_euid,
@@ -1075,6 +1179,7 @@ udisks_daemon_launch_spawned_job_sync (UDisksDaemon    *daemon,
                                           object,
                                           job_operation,
                                           job_started_by_uid,
+                                          no_inhibit,
                                           cancellable,
                                           run_as_uid,
                                           run_as_euid,
@@ -1095,6 +1200,7 @@ udisks_daemon_launch_spawned_job_sync (UDisksDaemon    *daemon,
  * @object: (allow-none): A #UDisksObject to add to the job or %NULL.
  * @job_operation: The operation for the job.
  * @job_started_by_uid: The user who started the job.
+ * @no_inhibit: Avoid placing system inhibitor lock.
  * @cancellable: A #GCancellable or %NULL.
  * @run_as_uid: The #uid_t to run the command as.
  * @run_as_euid: The effective #uid_t to run the command as.
@@ -1118,6 +1224,7 @@ udisks_daemon_launch_spawned_job_gstring_sync (UDisksDaemon    *daemon,
                                        UDisksObject    *object,
                                        const gchar     *job_operation,
                                        uid_t            job_started_by_uid,
+                                       gboolean         no_inhibit,
                                        GCancellable    *cancellable,
                                        uid_t            run_as_uid,
                                        uid_t            run_as_euid,
@@ -1150,6 +1257,7 @@ udisks_daemon_launch_spawned_job_gstring_sync (UDisksDaemon    *daemon,
                                           object,
                                           job_operation,
                                           job_started_by_uid,
+                                          no_inhibit,
                                           cancellable,
                                           run_as_uid,
                                           run_as_euid,
@@ -1229,6 +1337,7 @@ udisks_bd_thread_disable_progress (void)
  * @object: (allow-none): A #UDisksObject to add to the job or %NULL.
  * @job_operation: The operation for the job.
  * @job_started_by_uid: The user who started the job.
+ * @no_inhibit: Avoid placing system inhibitor lock.
  * @job_func: The function to run in another thread.
  * @user_data: User data to pass to @job_func.
  * @user_data_free_func: Function to free @user_data with or %NULL.
@@ -1245,6 +1354,7 @@ udisks_daemon_launch_threaded_job_sync (UDisksDaemon          *daemon,
                                         UDisksObject          *object,
                                         const gchar           *job_operation,
                                         uid_t                  job_started_by_uid,
+                                        gboolean               no_inhibit,
                                         UDisksThreadedJobFunc  job_func,
                                         gpointer               user_data,
                                         GDestroyNotify         user_data_free_func,
@@ -1261,6 +1371,7 @@ udisks_daemon_launch_threaded_job_sync (UDisksDaemon          *daemon,
                                            object,
                                            job_operation,
                                            job_started_by_uid,
+                                           no_inhibit,
                                            job_func,
                                            user_data,
                                            user_data_free_func,
@@ -1584,6 +1695,52 @@ udisks_daemon_find_block_by_device_file (UDisksDaemon *daemon,
   return ret;
 }
 
+/**
+ * udisks_daemon_find_block_by_device_file_and_symlinks:
+ * @daemon: A #UDisksDaemon.
+ * @device_file: A device file.
+ *
+ * Like udisks_daemon_find_block_by_device_file, but also checks all
+ * symlinks for the device.
+ *
+ * Returns: (transfer full): A #UDisksObject or %NULL if not found. Free with g_object_unref().
+ */
+UDisksObject *
+udisks_daemon_find_block_by_device_file_and_symlinks (UDisksDaemon *daemon,
+                                                      const gchar  *device_file)
+{
+  UDisksObject *ret = NULL;
+  GList *objects, *l;
+
+  objects = g_dbus_object_manager_get_objects (G_DBUS_OBJECT_MANAGER (daemon->object_manager));
+  for (l = objects; l != NULL; l = l->next)
+    {
+      UDisksObject *object = UDISKS_OBJECT (l->data);
+      UDisksBlock *block;
+      const gchar *const *symlinks;
+
+      block = udisks_object_peek_block (object);
+      if (block == NULL)
+        continue;
+
+      if (g_strcmp0 (udisks_block_get_device (block), device_file) == 0)
+        {
+          ret = g_object_ref (object);
+          goto out;
+        }
+
+      symlinks = udisks_block_get_symlinks (UDISKS_BLOCK (block));
+      if (symlinks && g_strv_contains (symlinks, device_file))
+        {
+          ret = g_object_ref (object);
+          goto out;
+        }
+    }
+ out:
+  g_list_free_full (objects, g_object_unref);
+  return ret;
+}
+
 /* ---------------------------------------------------------------------------------------------------- */
 
 /**
@@ -1653,7 +1810,7 @@ udisks_daemon_find_object (UDisksDaemon *daemon,
  *
  * Gets all D-Bus objects exported by @daemon.
  *
- * Returns: (transfer full) (element-type UDisksObject): A list of #UDisksObject instaces. The returned list should be freed with g_list_free() after each element has been freed with g_object_unref().
+ * Returns: (transfer full) (element-type UDisksObject): A list of #UDisksObject instances. The returned list should be freed with g_list_free() after each element has been freed with g_object_unref().
  */
 GList *
 udisks_daemon_get_objects (UDisksDaemon *daemon)
@@ -1682,7 +1839,7 @@ udisks_daemon_get_module_manager (UDisksDaemon *daemon)
  *
  * Gets the config manager used by @daemon.
  *
- * Returns: A #ConfigModuleManager. Do not free, the object is owned by @daemon.
+ * Returns: A #UDisksConfigManager. Do not free, the object is owned by @daemon.
  */
 UDisksConfigManager *
 udisks_daemon_get_config_manager (UDisksDaemon *daemon)
@@ -1754,12 +1911,38 @@ udisks_daemon_get_enable_tcrypt (UDisksDaemon *daemon)
   return daemon->enable_tcrypt;
 }
 
+/**
+ * udisks_daemon_get_uuid:
+ * @daemon: A #UDisksDaemon.
+ *
+ * Gets the UUID string specific to this @daemon instance.
+ *
+ * Returns: the UUID string. Do not free, the string is owned by @daemon.
+ */
+const gchar *
+udisks_daemon_get_uuid (UDisksDaemon *daemon)
+{
+  g_return_val_if_fail (UDISKS_IS_DAEMON (daemon), NULL);
+  return daemon->uuid;
+}
+
 /* ---------------------------------------------------------------------------------------------------- */
 
+/**
+ * udisks_daemon_get_parent_for_tracking:
+ * @daemon: A #UDisksDaemon.
+ * @path: object path of a child to find parent of
+ * @uuid: a pointer to return parent UUID string
+ *
+ * Finds parent block device and returns its object path and UUID.
+ * If the return value is %NULL, the value of @uuid has not been changed.
+ *
+ * Returns: (transfer full): object path of the parent device. Free with g_free().
+ */
 gchar *
 udisks_daemon_get_parent_for_tracking (UDisksDaemon  *daemon,
                                        const gchar   *path,
-                                       gchar        **uuid_ret)
+                                       gchar        **uuid)
 {
   const gchar *parent_path = NULL;
   const gchar *parent_uuid = NULL;
@@ -1768,7 +1951,6 @@ udisks_daemon_get_parent_for_tracking (UDisksDaemon  *daemon,
   UDisksObject *crypto_object = NULL;
   UDisksObject *mdraid_object = NULL;
   UDisksObject *table_object = NULL;
-  GList *track_parent_funcs;
 
   UDisksBlock *block;
   UDisksBlock *crypto_block;
@@ -1776,6 +1958,10 @@ udisks_daemon_get_parent_for_tracking (UDisksDaemon  *daemon,
   UDisksPartition *partition;
   UDisksBlock *table_block;
 
+  GList *modules;
+  GList *l;
+  gchar *path_ret = NULL;
+
   object = udisks_daemon_find_object (daemon, path);
   if (object == NULL)
     goto out;
@@ -1836,21 +2022,24 @@ udisks_daemon_get_parent_for_tracking (UDisksDaemon  *daemon,
 
   if (parent_path)
     {
-      if (uuid_ret)
-        *uuid_ret = g_strdup (parent_uuid);
+      if (uuid)
+        *uuid = g_strdup (parent_uuid);
       return g_strdup (parent_path);
     }
 
-  track_parent_funcs = udisks_module_manager_get_track_parent_funcs (daemon->module_manager);
-  while (track_parent_funcs)
+  modules = udisks_module_manager_get_modules (daemon->module_manager);
+  for (l = modules; l != NULL; l = l->next)
     {
-      UDisksTrackParentFunc func = track_parent_funcs->data;
-      gchar *path_ret = func (daemon, path, uuid_ret);
-      if (path_ret)
-        return path_ret;
+      UDisksModule *module = l->data;
 
-      track_parent_funcs = track_parent_funcs->next;
+      path_ret = udisks_module_track_parent (module, path, uuid);
+      if (path_ret)
+        break;
     }
+  g_list_free_full (modules, g_object_unref);
+
+  if (path_ret)
+    return path_ret;
 
   return NULL;
 }
