@@ -324,6 +324,14 @@ has_option (const gchar *options,
   return ret;
 }
 
+static gchar *
+label_to_safe_dm_name (const gchar *label)
+{
+  if (strlen (label) >= 128)
+    return g_strndup (label, 127);
+  return g_strdelimit (g_strdup (label), "/ ", '_');
+}
+
 /* ---------------------------------------------------------------------------------------------------- */
 
 /* runs in thread dedicated to handling @invocation */
@@ -365,6 +373,7 @@ handle_unlock (UDisksEncrypted        *encrypted,
   gboolean handle_as_tcrypt;
   void *open_func;
   const gchar *uuid = NULL;
+  const gchar *label = NULL;
 
   object = udisks_daemon_util_dup_object (encrypted, &error);
   if (object == NULL)
@@ -524,19 +533,26 @@ handle_unlock (UDisksEncrypted        *encrypted,
   if (is_in_crypttab && crypttab_name != NULL)
     name = g_strdup (crypttab_name);
   else {
-    if (is_luks)
-      name = g_strdup_printf ("luks-%s", udisks_block_get_id_uuid (block));
-    else if (is_bitlk)
-      {
-        uuid = udisks_block_get_id_uuid (block);
-        if (uuid && g_strcmp0 (uuid, "") != 0)
-          name = g_strdup_printf ("bitlk-%s", uuid);
-        else
-          name = g_strdup_printf ("bitlk-%" G_GUINT64_FORMAT, udisks_block_get_device_number (block));
-      }
+    label = udisks_block_get_id_label (block);
+    if (label)
+      name = label_to_safe_dm_name (label);
     else
-      /* TCRYPT devices don't have a UUID, so we use the device number instead */
-      name = g_strdup_printf ("tcrypt-%" G_GUINT64_FORMAT, udisks_block_get_device_number (block));
+      {
+        if (is_luks)
+          name = g_strdup_printf ("luks-%s", udisks_block_get_id_uuid (block));
+        else if (is_bitlk)
+          {
+            uuid = udisks_block_get_id_uuid (block);
+            if (uuid && g_strcmp0 (uuid, "") != 0)
+              name = g_strdup_printf ("bitlk-%s", uuid);
+            else
+              name = g_strdup_printf ("bitlk-%" G_GUINT64_FORMAT, udisks_block_get_device_number (block));
+          }
+        else
+          /* TCRYPT devices don't have a UUID, so we use the device number instead */
+          name = g_strdup_printf ("tcrypt-%" G_GUINT64_FORMAT, udisks_block_get_device_number (block));
+      }
+
   }
 
   /* save old encryption type to be able to restore it */
@@ -580,6 +596,7 @@ handle_unlock (UDisksEncrypted        *encrypted,
                                                object,
                                                "encrypted-unlock",
                                                caller_uid,
+                                               FALSE,
                                                open_func,
                                                &data,
                                                NULL, /* user_data_free_func */
@@ -799,6 +816,7 @@ udisks_linux_encrypted_lock (UDisksLinuxEncrypted   *encrypted,
                                                object,
                                                "encrypted-lock",
                                                caller_uid,
+                                               FALSE,
                                                close_func,
                                                &data,
                                                NULL, /* user_data_free_func */
@@ -983,6 +1001,7 @@ handle_change_passphrase (UDisksEncrypted        *encrypted,
                                                object,
                                                "encrypted-modify",
                                                caller_uid,
+                                               FALSE,
                                                luks_change_key_job_func,
                                                &data,
                                                NULL, /* user_data_free_func */
@@ -1147,6 +1166,7 @@ handle_resize (UDisksEncrypted       *encrypted,
                                          UDISKS_OBJECT (object),
                                          "encrypted-resize",
                                          caller_uid,
+                                         FALSE,
                                          NULL);
   if (job == NULL)
     {
@@ -1288,6 +1308,7 @@ handle_convert (UDisksEncrypted       *encrypted,
                                          UDISKS_OBJECT (object),
                                          "encrypted-convert",
                                          caller_uid,
+                                         FALSE,
                                          NULL);
   if (job == NULL)
     {
@@ -1341,6 +1362,7 @@ handle_header_backup (UDisksEncrypted       *encrypted,
     UDisksBlock *block;
     UDisksDaemon *daemon;
     UDisksState *state = NULL;
+    const gchar *action_id;
     uid_t caller_uid;
     GError *error = NULL;
     UDisksBaseJob *job = NULL;
@@ -1377,10 +1399,29 @@ handle_header_backup (UDisksEncrypted       *encrypted,
         goto out;
       }
 
+    action_id = "org.freedesktop.udisks2.open-device";
+    if (udisks_block_get_hint_system (block))
+      action_id = "org.freedesktop.udisks2.open-device-system";
+
+    if (!udisks_daemon_util_check_authorization_sync (daemon,
+                                                      object,
+                                                      action_id,
+                                                      options,
+                                                      /* Translators: Shown in authentication dialog when backing up
+                                                       * a LUKS header of a device.
+                                                       *
+                                                       * Do not translate $(device.name), it's a placeholder and will
+                                                       * be replaced by the name of the drive/device in question
+                                                       */
+                                                      N_("Authentication is required to back up the encrypted header of $(device.name)"),
+                                                      invocation))
+      goto out;
+
     job = udisks_daemon_launch_simple_job (daemon,
                                            UDISKS_OBJECT (object),
                                            "encrypted-header-backup",
                                            caller_uid,
+                                           FALSE,
                                            NULL);
     if (job == NULL)
       {
