@@ -1185,6 +1185,37 @@ get_block_objects (UDisksManager *manager,
   return ret;
 }
 
+static GSList*
+get_drive_objects (UDisksManager *manager,
+                   guint         *num_drives)
+{
+  UDisksLinuxManager *linux_manager = UDISKS_LINUX_MANAGER (manager);
+  GDBusObjectManagerServer *object_manager = NULL;
+  GList *objects = NULL;
+  GList *objects_p = NULL;
+  GSList *ret = NULL;
+
+  object_manager = udisks_daemon_get_object_manager (linux_manager->daemon);
+  objects = g_dbus_object_manager_get_objects (G_DBUS_OBJECT_MANAGER (object_manager));
+
+  for (objects_p = objects; objects_p != NULL; objects_p = objects_p->next)
+    {
+      UDisksObject *object = UDISKS_OBJECT (objects_p->data);
+      UDisksDrive *drive;
+
+      drive = udisks_object_get_drive (object);
+      if (drive != NULL)
+        {
+          ret = g_slist_prepend (ret, drive);
+          (*num_drives)++;
+        }
+    }
+
+  g_list_free_full (objects, g_object_unref);
+  ret = g_slist_reverse (ret);
+  return ret;
+}
+
 static gboolean
 handle_get_block_devices (UDisksManager         *object,
                           GDBusMethodInvocation *invocation,
@@ -1215,6 +1246,37 @@ handle_get_block_devices (UDisksManager         *object,
   return TRUE;  /* returning TRUE means that we handled the method invocation */
 }
 
+static gboolean
+handle_get_drives (UDisksManager         *object,
+                   GDBusMethodInvocation *invocation,
+                   GVariant              *arg_options)
+{
+  GSList *drives = NULL;
+  GSList *drives_p = NULL;
+  const gchar **drive_paths = NULL;
+  guint num_drives = 0;
+  guint i = 0;
+
+  drives = get_drive_objects (object, &num_drives);
+  drive_paths = g_new0 (const gchar *, num_drives + 1);
+
+  for (drives_p = drives; drives_p != NULL; drives_p = drives_p->next)
+    {
+      GDBusObject *drive_object = g_dbus_interface_get_object (G_DBUS_INTERFACE (drives_p->data));
+      if (drive_object)
+        drive_paths[i++] = g_dbus_object_get_object_path (drive_object);
+    }
+
+  udisks_manager_complete_get_drives (object,
+                                      invocation,
+                                      drive_paths);
+
+  g_free (drive_paths);
+  g_slist_free_full (drives, g_object_unref);
+
+  return TRUE;  /* returning TRUE means that we handled the method invocation */
+}
+
 static inline gboolean
 match_id_format (UDisksLinuxBlock *block, const gchar *key, const gchar *val)
 {
@@ -1323,4 +1385,5 @@ manager_iface_init (UDisksManagerIface *iface)
   iface->handle_can_repair = handle_can_repair;
   iface->handle_get_block_devices = handle_get_block_devices;
   iface->handle_resolve_device = handle_resolve_device;
+  iface->handle_get_drives = handle_get_drives;
 }
