@@ -377,7 +377,8 @@ udisks_client_get_object_info_for_mdraid (UDisksClient     *client,
   gchar *size_str = NULL;
   const gchar *name;
   const gchar *level;
-  gchar *s;
+  const gchar *s;
+  gchar *desc;
 
   block = udisks_client_get_block_for_mdraid (client, mdraid);
 
@@ -416,10 +417,10 @@ udisks_client_get_object_info_for_mdraid (UDisksClient     *client,
        *              The %u is the partition number.
        *              The %s is the description for the drive (e.g. "2 TB RAID-5").
        */
-      s = g_strdup_printf (C_("part-raid", "Partition %u of %s"),
-                           udisks_partition_get_number (partition), info->description);
+      desc = g_strdup_printf (C_("part-raid", "Partition %u of %s"),
+                              udisks_partition_get_number (partition), info->description);
       g_free (info->description);
-      info->description = s;
+      info->description = desc;
     }
 
   if (strlen (info->name) > 0)
@@ -452,9 +453,8 @@ udisks_client_get_object_info_for_mdraid (UDisksClient     *client,
       if (block != NULL)
         {
           /* Translators: String used for one-liner description of running RAID array w/o a name.
-           *              The first %s is the array name (e.g. "AlphaGo").
-           *              The second %s is the size and level (e.g. "2 TB RAID-5").
-           *              The third %s is the special device file (e.g. "/dev/sda").
+           *              The first %s is the size and level (e.g. "2 TB RAID-5").
+           *              The second %s is the special device file (e.g. "/dev/sda").
            */
           info->one_liner = g_strdup_printf (C_("one-liner-mdraid-no-name-running", "%s — %s"),
                                              info->description,
@@ -463,8 +463,7 @@ udisks_client_get_object_info_for_mdraid (UDisksClient     *client,
       else
         {
           /* Translators: String used for one-liner description of non-running RAID array w/o a name.
-           *              The first %s is the array name (e.g. "AlphaGo").
-           *              The second %s is the size and level (e.g. "2 TB RAID-5").
+           *              The %s is the size and level (e.g. "2 TB RAID-5").
            */
           info->one_liner = g_strdup_printf (C_("one-liner-mdraid-no-name-not-running", "%s"),
                                              info->description);
